
__version__ = '0.1.1'
__author__ = 'Juan Batiz-Benet'
__email__ = 'juan@benet.ai'
__doc__ = '''
pylru datastore implementation.

Tested with:
  * datastore 0.3.0
  * pylru 1.0.5

'''

import pylru
import datastore.core


class LRUCacheDatastore(datastore.Datastore):
  '''Represents an LRU cache datastore, backed by pylru.

  Hello World:

      >>> import datastore.pylru
      >>>
      >>> ds = datastore.pylru.LRUCacheDatastore(100)
      >>>
      >>> hello = datastore.Key('hello')
      >>> ds.put(hello, 'world')
      >>> ds.contains(hello)
      True
      >>> ds.get(hello)
      'world'
      >>> ds.delete(hello)
      >>> ds.get(hello)
      None

  '''

  def __init__(self, size):
    self._cache = pylru.lrucache(size)

  def __len__(self):
    return len(self._cache)

  def clear(self):
    self._cache.clear()

  def get(self, key):
    '''Return the object named by key.'''
    try:
      return self._cache[key]
    except KeyError, e:
      return None

  def put(self, key, value):
    '''Stores the object.'''
    self._cache[key] = value

  def delete(self, key):
    '''Removes the object.'''
    if key in self._cache:
      del self._cache[key]

  def contains(self, key):
    '''Returns whether the object is in this datastore.'''
    return key in self._cache

  def query(self, query):
    '''Returns a sequence of objects matching criteria expressed in `query`'''
    # entire dataset already in memory, so ok to apply query naively
    return query(self._cache.values())




import unittest

from datastore import Key
from datastore import DictDatastore
from datastore import NullDatastore
from datastore import CacheShimDatastore
from datastore.core.test.test_basic import TestDatastore

from . import LRUCacheDatastore

class TestLRUCacheDatastore(TestDatastore):


  def subtest_cache(self, lrus):

    lru1 = lrus[0]
    lru2 = lrus[1]
    lru3 = lrus[2]

    for value in range(0, 3000):
      key = Key('/LRU/%d' % value)
      for lru in lrus:
        self.assertFalse(lru.contains(key))

        lru.put(key, value)

        self.assertTrue(lru.contains(key))
        self.assertEqual(lru.get(key), value)

    if hasattr(lru1, '__len__'):
      self.assertEqual(len(lru1), 1000)
      self.assertEqual(len(lru2), 2000)
      self.assertEqual(len(lru3), 3000)

    for value in range(0, 3000):
      key = Key('/LRU/%d' % value)
      self.assertEqual(lru1.contains(key), value >= 2000)
      self.assertEqual(lru2.contains(key), value >= 1000)
      self.assertTrue(lru3.contains(key))

    if hasattr(lru1, 'clear'):
      lru1.clear()
      lru2.clear()
      lru3.clear()

      self.assertEqual(len(lru1), 0)
      self.assertEqual(len(lru2), 0)
      self.assertEqual(len(lru3), 0)

      self.subtest_simple(lrus)


  def test_lru(self):

    lru1 = LRUCacheDatastore(1000)
    lru2 = LRUCacheDatastore(2000)
    lru3 = LRUCacheDatastore(3000)
    lrus = [lru1, lru2, lru3]
    self.subtest_cache(lrus)


  def test_shim(self):

    lru1 = LRUCacheDatastore(1000)
    lru2 = LRUCacheDatastore(2000)
    lru3 = LRUCacheDatastore(3000)

    ds1 = CacheShimDatastore(NullDatastore(), cache=lru1)
    ds2 = CacheShimDatastore(NullDatastore(), cache=lru2)
    ds3 = CacheShimDatastore(NullDatastore(), cache=lru3)

    lrus = [ds1, ds2, ds3]
    self.subtest_cache(lrus)

    lru1.clear()
    lru2.clear()
    lru3.clear()

    class NullMinusQueryDatastore(NullDatastore):
      def query(self, query):
        raise NotImplementedError

    ds1 = CacheShimDatastore(NullMinusQueryDatastore(), cache=lru1)
    ds2 = CacheShimDatastore(NullMinusQueryDatastore(), cache=lru2)
    ds3 = CacheShimDatastore(NullMinusQueryDatastore(), cache=lru3)

    lrus = [ds1, ds2, ds3]
    self.subtest_simple(lrus)

    ds1 = CacheShimDatastore(DictDatastore(), cache=lru1)
    ds2 = CacheShimDatastore(DictDatastore(), cache=lru2)
    ds3 = CacheShimDatastore(DictDatastore(), cache=lru3)

    lrus = [ds1, ds2, ds3]
    self.subtest_simple(lrus)




if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python

import re
from setuptools import setup, find_packages

pkgname = 'datastore.pylru'

# gather the package information
main_py = open('datastore/pylru/__init__.py').read()
metadata = dict(re.findall("__([a-z]+)__ = '([^']+)'", main_py))
packages = filter(lambda p: p.startswith('datastore'), find_packages())

# convert the readme to pypi compatible rst
try:
  try:
    import pypandoc
    readme = pypandoc.convert('README.md', 'rst')
  except ImportError:
    readme = open('README.md').read()
except:
  print 'something went wrong reading the README.md file.'
  readme = ''

setup(
  name=pkgname,
  version=metadata['version'],
  description='pylru datastore implementation',
  long_description=readme,
  author=metadata['author'],
  author_email=metadata['email'],
  url='http://github.com/datastore/datastore.pylru',
  keywords=[
    'datastore',
    'pylru',
  ],
  packages=packages,
  namespace_packages=['datastore'],
  install_requires=['datastore>=0.3.3', 'pylru>=1.0.5'],
  test_suite='datastore.pylru.test',
  license='MIT License',
  classifiers=[
    'Topic :: Database',
    'Topic :: Database :: Front-Ends',
  ]
)
